import QtQuick 2.5
import Fly.Components 1.0
import Qt.labs.folderlistmodel 2.1
import Fly.Gallery 1.0 as GalleryPrivate //QTBUG-27645 workaround


Rectangle {
    id: galleryPage

    property real maxZoom: 10
    property alias folder: fileSystemModel.folder
    property alias parentFolder: fileSystemModel.parentFolder
    property alias count: gridViewProxyModel.count
    property alias sortField: fileSystemModel.sortField
    property alias sortReversed: fileSystemModel.sortReversed
    readonly property alias goBackAvailable: grid.goBackAvailable
    property var selectedPaths: []
    property int columns: 3
    property bool showFileNames: false
    property alias showDirs: fileSystemModel.showDirs
    readonly property alias currentFileName: list.currentFileName
    readonly property alias currentFilePath: list.currentFilePath
    readonly property bool viewByOne: state === "view-by-one"
    property bool fullView : false
    property bool showVideos: true
    property bool showDocuments: false
    property bool autoTransform: false
    property bool useFullScreenMode: false
    property alias animation: list.animation
    property alias slideShow: list.slideShow
    property alias slideShowInterval: slideShowTimer.interval
    property alias maxImageSize: grid.maxImageSize
    property bool exitOnDoubleClick: false
    signal playVideoRequest(url mediaPath, var playlist)
    signal openDocumentRequest(url documentPath)
    signal pressAndHold

    color: styles.colors.black

    state: "view-all"

    states: [
        State {
            name: "view-all"
            PropertyChanges { target: grid; visible: true }
            PropertyChanges { target: list; visible: false }
            PropertyChanges { target: galleryPage; fullView: false }
        },
        State {
            name: "view-by-one"
            PropertyChanges { target: grid; visible: false }
            PropertyChanges { target: list; visible: true }
        }
    ]

    function setSelectable(selectable) {
        setViewAllState();
        if(selectable && !grid.selectable)
            selectedPaths = [];
        grid.selectable = selectable;
    }

    function setViewAllState() {
        if(state === "view-by-one") {
            state = "view-all";
            root.popFunctionFromBackStack();
            root.showNormal();
        }
    }

    function setViewByOneState() {
        if(state === "view-all") {
            state = "view-by-one";
            root.addFunctionToBackStack(setViewAllState);
            fullView = true;
            list.currentFileName = String(fullViewProxyModel.get(list.currentIndex, "fileName"));
            list.currentFilePath = String(fullViewProxyModel.get(list.currentIndex, "filePath"));
            if(useFullScreenMode)
                root.showFullScreen();
        }
    }

    function isImage(path) {
        return GalleryUtils.isImage(path);
    }

    function isVideo(path) {
        return GalleryUtils.isVideo(path);
    }

    function isDocument(path) {
        return GalleryUtils.isDocument(path);
    }

    function refresh() {
        //hack to refresh FolderListModel
        var folder = gridViewProxyModel.folder;
        fileSystemModel.folder = "/";
        fileSystemModel.folder = folder;
    }

    function goBack() {
        if(grid.history.length > 0) {
            var tmp = grid.history;
            folder = tmp.pop();
            grid.history = tmp;

            if(grid.history.length === 0)
                root.popFunctionFromBackStack();
        }
    }

    function clearBackHistory() {
        if(grid.history.length > 0) {
            var tmp = [];
            grid.history = tmp;
            root.popFunctionFromBackStack();
        }
    }

    GalleryPrivate.FileOps {
        id: fileOps
    }

    FolderListModel {
        id: fileSystemModel
        nameFilters: GalleryUtils.pickFilters(true, showVideos, showDocuments);
        showDirs: true
        showDirsFirst: true
    }

    GalleryPrivate.DirectoryFilterProxyModel {
        id: gridViewProxyModel
        source: fileSystemModel
        imageFilters: GalleryUtils.imageFilters()
    }

    GalleryPrivate.DirectoryFilterProxyModel {
        id: fullViewProxyModel
        source: fileSystemModel
        showDirs: false
        showVideos: false
        imageFilters: GalleryUtils.imageFilters()
    }

    Connections {
        target: fileSystemModel
        onFolderChanged: {
            grid.selectable = false;
            selectedPaths = [];
            setViewAllState();
            if(count > 0) {
                list.currentIndex = 0;
                list.currentFileName = String(fullViewProxyModel.get(list.currentIndex, "fileName"));
                list.currentFilePath = String(fullViewProxyModel.get(list.currentIndex, "filePath"));
            } else {
                list.currentIndex = -1;
                list.currentFileName.length = 0;
                list.currentFilePath.length = 0;
            }
        }
    }

    GridView {
        id: grid
        property bool selectable: false
        property bool goBackAvailable: history.length !== 0
        property var history: []
        property int maxImageSize: 30 * 1048576 //30 MiB
        enabled: visible
        anchors.fill: parent
        model: gridViewProxyModel
        cacheBuffer: cellHeight * 2
        cellWidth: Math.floor(width / galleryPage.columns);
        cellHeight: Math.floor(cellWidth * 0.7) // TODO depend on current view

        delegate: Loader {
            asynchronous: true
            sourceComponent: ImageComponent {
                id: imageComponent
                readonly property int _index: index
                readonly property url _mediaPath: fileURL
                width: grid.cellWidth - 1
                height: grid.cellHeight - 1
                autoTransform: galleryPage.autoTransform

                Component.onCompleted: {
                    title = fileName
                    if(fileIsDir) {
                        mediaSource = "qrc:/icons/folder_gallery.svg";
                        var p = gridViewProxyModel.getFirstFilePath(index);
                        if(GalleryUtils.isVideo(p)) {
                            thumbnailSource = "qrc:/icons/video-file-thumb.svg";
                        } else if(GalleryUtils.isImage(p)) {
                            thumbnailSource = fileOps.getFileSize(p) > grid.maxImageSize ?
                                        "qrc:/icons/image-file-thumb.svg" : p;
                        } else if(GalleryUtils.isDocument(p)) {
                            if(fileOps.getFileSize(p) > grid.maxImageSize)
                                thumbnailSource = "qrc:/icons/pdf-file.svg";
                            else
                                thumbnailDocumentSource = p;
                        }
                        selected = false;
                        selectable = false;
                        title = fileName + " (" + gridViewProxyModel.getElementsCount(index) + ")"
                        titleVisible = true;
                        thumbnailFillMode = Image.PreserveAspectFit;
                    }
                    else if(GalleryUtils.isImage(filePath)) {
                        if(fileSize < grid.maxImageSize) {
                            mediaSource = fileURL;
                            if(fileSize < 10240) //do not crop small images (less than 10 KiB)
                                fillMode = Image.PreserveAspectFit;
                            else
                                fillMode = Image.PreserveAspectCrop;
                        } else {
                            mediaSource = "qrc:/icons/image-file.svg"
                            fillMode = Image.PreserveAspectFit;
                        }
                        titleVisible = Qt.binding(function() { return showFileNames });
                        selectable = grid.selectable;
                        if(selectable)
                            selected = selectedPaths.indexOf(filePath) != -1;

                    }
                    else if(GalleryUtils.isDocument(filePath)) {
                        if(fileSize < grid.maxImageSize) {
                            documentSource = fileURL;
                        } else {
                            mediaSource = "qrc:/icons/pdf-file.svg"
                            fillMode = Image.PreserveAspectFit;
                        }
                        selectable = grid.selectable;
                        if(selectable)
                            selected = selectedPaths.indexOf(filePath) != -1;
                    }
                    else {
                        mediaSource = "qrc:/icons/video-file.svg";
                        fillMode = Image.PreserveAspectFit;
                        selectable = grid.selectable;
                        if(selectable)
                            selected = selectedPaths.indexOf(filePath) != -1;
                    }
                }

                onClicked: {
                    if(galleryPage.state === "view-all") {
                        if(fileIsDir) {
                            if(grid.history.length === 0) {
                                root.addFunctionToBackStack(gallery.goBack);
                            }
                            var tmp = grid.history;
                            tmp.push(fileSystemModel.folder)
                            grid.history = tmp;
                            fileSystemModel.folder = fileURL;
                        }
                        else if(GalleryUtils.isVideo(_mediaPath)) {
                            var playlist = [];
                            for(var i = 0; i < gridViewProxyModel.count; ++i) {
                                var path = gridViewProxyModel.get(i, "fileURL");
                                if(GalleryUtils.isVideo(path))
                                    playlist.push(path);
                            }
                            playVideoRequest(_mediaPath, playlist);
                        } else if(GalleryUtils.isDocument(_mediaPath)) {
                            openDocumentRequest(_mediaPath);
                        } else {
                            //Scroll to current file
                            for(var j = 0; j < fullViewProxyModel.count; j++) {
                                if(fileURL === fullViewProxyModel.get(j, "fileURL"))
                                    list.positionViewAtIndex(j, ListView.SnapPosition);
                            }
                            setViewByOneState();
                        }
                    }
                    else
                        setViewAllState();
                }

                Connections {
                    target: grid
                    onSelectableChanged: {
                        if(!fileIsDir)
                            imageComponent.selectable = grid.selectable
                    }
                }
                Connections {
                    target: imageComponent
                    onSelectedChanged: {
                        var tmp = selectedPaths;
                        if(imageComponent.selected) {
                            tmp.push(filePath);
                        } else {
                            var index = selectedPaths.indexOf(filePath);
                            tmp.splice(index, 1);
                        }
                        //sends notification about property change
                        //see http://stackoverflow.com/questions/19583234/qml-binding-to-an-array-element
                        selectedPaths = tmp;
                    }
                }
            }
        }

        PinchArea {
            anchors.fill: parent

            onPinchUpdated: {
                var c = galleryPage.columns;
                if(pinch.scale > 1.0)
                    c++;
                else
                    c--;
                c = Math.max(3, Math.min(c, 8))
                galleryPage.columns = c;
            }

            MouseArea {
                anchors.fill: parent
                propagateComposedEvents: true
                enabled: false

                onPressAndHold: galleryPage.pressAndHold();

                Component.onCompleted: {
                    //The problem is that MouseArea eats first press event. This hack prevents that issue.
                    //TODO fill bugreport
                    enabled = true;
                }
            }
        }
    }

    ListView {
        id: list
        property string currentFileName
        property string currentFilePath
        property bool slideShow: false
        property bool animation: true

        anchors.fill: parent
        model: fullViewProxyModel
        snapMode: ListView.SnapOneItem
        orientation: ListView.Horizontal
        preferredHighlightBegin: 0.5; preferredHighlightEnd: 0.5
        highlightRangeMode: ListView.StrictlyEnforceRange
        enabled: visible
        interactive: animation

        delegate: Loader {
            id: bigImageLoader
            asynchronous: true
            width: ListView.view.width; height: ListView.view.height;
            readonly property bool _isCurrentItem: ListView.isCurrentItem
            sourceComponent: BigImageComponent {
                id: bigImageComponent
                autoTransform: galleryPage.autoTransform
                exitOnDoubleClick: gallery.exitOnDoubleClick
                swipe: !list.animation

                mediaPath: {
                    if(fileIsDir)
                        return "qrc:/icons/folder.svg";
                    if(GalleryUtils.isImage(filePath))
                        return visible ? fileURL : "";
                    else
                        return "qrc:/icons/video-file.svg";
                }

                onClicked: {
                    if(galleryPage.state === "view-all")
                    {
                        list.positionViewAtIndex(_index, ListView.SnapPosition);
                        setViewByOneState();
                    } else if(galleryPage.state === "view-by-one") {
                        galleryPage.fullView = !galleryPage.fullView;
                    }
                }

                onNextRequest: {
                    if(list.currentIndex >= list.count - 1)
                        list.positionViewAtIndex(0, ListView.Beginning);
                    else
                        list.positionViewAtIndex(list.currentIndex + 1, ListView.Beginning);
                }

                onPreviousRequest: {
                    if(list.currentIndex === 0)
                        list.positionViewAtIndex(list.count - 1, ListView.Beginning);
                    else
                        list.positionViewAtIndex(list.currentIndex - 1, ListView.Beginning);
                }
            }
            on_IsCurrentItemChanged: bigImageLoader.item.state = "view-image"
        }

        onCurrentIndexChanged: {
            currentFileName = String(fullViewProxyModel.get(currentIndex, "fileName"));
            currentFilePath = String(fullViewProxyModel.get(currentIndex, "filePath"));
        }

        Shortcut {
            sequence: "space"
            enabled: list.visible
            onActivated: {
                if(list.currentIndex >= list.count - 1)
                    list.positionViewAtIndex(0, ListView.Beginning);
                else if(animation)
                    list.currentIndex++;
                else
                    list.positionViewAtIndex(list.currentIndex+1, ListView.Beginning);
            }
        }

        Shortcut {
            sequence: "esc"
            enabled: list.visible
            onActivated: setViewAllState()
        }

        Timer {
            id: slideShowTimer
            running: list.slideShow && list.visible
            repeat: true
            interval: 5000

            onTriggered: {
                if(list.currentIndex >= list.count - 1)
                    list.positionViewAtIndex(0, ListView.Beginning);
                else
                    list.currentIndex++;
            }
        }
    }
}
