import QtQuick 2.5
import Fly.Components 1.0
import Fly.Gallery 1.0

Item {
    property alias mediaSource: previewImage.source
    property alias thumbnailSource: thumbnailImage.source
    property alias thumbnailFillMode: thumbnailImage.fillMode
    property alias documentSource: pdfPreviewLoader.source
    property alias thumbnailDocumentSource: pdfThumbnailLoader.source
    property alias fillMode: previewImage.fillMode
    property alias title: titleText.text
    property alias titleVisible: titleText.visible
    property alias autoTransform: previewImage.autoTransform

    property bool selected: false
    property bool selectable: false

    signal clicked

    onSelectableChanged: {
        if(!selectable)
            selected = false;
    }

    Rectangle
    {
        id: selectionBackground
        anchors.fill: parent
        visible: parent.selectable
        color: styles.colors.black
        opacity: 0.4
        z: 0.5
    }

    Image {
        id: previewImage
        anchors.fill: parent
        fillMode: Image.PreserveAspectFit
        height: titleText.visible ? parent.height - titleText.height : parent.height
        asynchronous: true
        sourceSize: Qt.size(4*height,0)
        cache: true
        smooth: false
        autoTransform: true
        visible: source.toString().length > 0

        Image {
            id: thumbnailImage
            anchors { fill: parent; margins: { parent.height / 3,  parent.height / 3,  parent.height / 3,  parent.height / 3 } }
            visible: source.toString().length > 0
            sourceSize: Qt.size(4*height,0)
            asynchronous: true
            cache: true
            smooth: false
            autoTransform: true
        }

        Item {
            id:  pdfThumbnail
            anchors { fill: parent; margins: { parent.height / 3,  parent.height / 3,  parent.height / 3,  parent.height / 3 } }
            visible: pdfThumbnailLoader.source.toString().length > 0

            PdfPreviewLoader {
                id: pdfThumbnailLoader
            }

            Rectangle {
                id: pdfThumbnailBackground
                anchors { top: pdfThumbnail.top; bottom: pdfThumbnail.bottom; horizontalCenter: pdfThumbnail.horizontalCenter }
                width: pdfThumbnailImage.paintedWidth
                color: "white"
            }

            Image {
                id: pdfThumbnailImage
                anchors.fill: parent
                fillMode: Image.PreserveAspectFit
                cache: false
                source: pdfThumbnailLoader.previewUrl
                onWidthChanged: pdfThumbnailLoader.renderPage(0, Qt.size(width, 0))
            }
        }
    }

    Rectangle {
        id: pdfPreview
        anchors.fill: parent
        color: "white"
        visible: pdfPreviewLoader.source.toString().length > 0

        PdfPreviewLoader {
            id: pdfPreviewLoader
        }

        Image {
            anchors.fill: parent
            fillMode: Image.PreserveAspectFit
            cache: false
            source: pdfPreviewLoader.previewUrl
            onWidthChanged: pdfPreviewLoader.renderPage(0, Qt.size(width, 0))
        }
    }

    Rectangle {
        id: titleTextBackground
        visible: titleText.visible
        color: styles.colors.black
        opacity: 0.4
        z: 0.1
        anchors { bottom: parent.bottom; left: parent.left; right: parent.right }
        height: 25 * dpiRatio
    }

    Text {
        id: titleText
        color: styles.colors.white
        anchors.fill: titleTextBackground
        x: (parent.width - width) / 2
        elide: Text.ElideRight
        style: Text.Normal
        horizontalAlignment: Text.AlignHCenter
        verticalAlignment: Text.AlignVCenter
        font.pointSize: 12 * dpiRatio
        width: parent.width - 10 * dpiRatio
        z: 0.2
    }

    Image {
        anchors { horizontalCenter:   parent.horizontalCenter }
        visible: parent.selectable
        source: parent.selected ? "qrc:/icons/checked-checkbox.svg" : "qrc:/icons/unchecked-checkbox.svg";
        sourceSize: Qt.size(parent.height * 0.6, parent.height * 0.6)
        opacity: 1
        z: 1
    }

    MouseArea {
        id: imageMouseArea
        anchors.fill: parent
        onClicked: {
            if(selectable)
                parent.selected = !parent.selected;
            else
                parent.clicked();
        }
    }
}
