import QtQuick 2.5
import Fly.Components 1.0
import Fly.Gallery 1.0 as GalleryPrivate //QTBUG-27645 workaround

Item {
    id: pdfViewer
    property alias source: pdfModel.source
    property bool showPageNumbers: true
    readonly property real maxScale: 10.0
    readonly property real minScale: 0.5
    property alias animation: listView.animation
    property bool exitOnDoubleClick: false
    signal clicked

    GalleryPrivate.PdfModel {
        id: pdfModel
    }

    ListView {
        id: listView
        property bool animation: true
        anchors.fill: parent
        spacing: 2
        snapMode: ListView.SnapOneItem
        orientation: ListView.Horizontal
        preferredHighlightBegin: 0.5; preferredHighlightEnd: 0.5
        highlightRangeMode: ListView.StrictlyEnforceRange
        model: pdfModel
        currentIndex: 0
        interactive: animation

        delegate: Loader {
            id: pdfLoader
            asynchronous: true
            width: ListView.view.width; height: ListView.view.height;
            readonly property bool _isCurrentItem: ListView.isCurrentItem
            sourceComponent: PdfComponent {
                model: pdfModel
                page: index
                showPageNumber: pdfViewer.showPageNumbers
                exitOnDoubleClick: pdfViewer.exitOnDoubleClick
                swipe: !listView.animation
                onClicked: pdfViewer.clicked()

                onNextRequest: {
                    if(listView.currentIndex >= listView.count - 1)
                        listView.positionViewAtIndex(0, ListView.Beginning);
                    else
                        listView.positionViewAtIndex(listView.currentIndex + 1, ListView.Beginning);
                }

                onPreviousRequest: {
                    if(listView.currentIndex === 0)
                        listView.positionViewAtIndex(listView.count - 1, ListView.Beginning);
                    else
                        listView.positionViewAtIndex(listView.currentIndex - 1, ListView.Beginning);
                }
            }
            on_IsCurrentItemChanged: pdfLoader.item.state = "view-pdf"
        }
    }

    FlyColoredImage {
        id: stateIcon
        visible: pdfModel.loading
        imagePath: "qrc:/icons/state_loading.svg"
        anchors { centerIn: parent }
        height: 150 * dpiRatio
        width: height
        antialiasing: true

        RotationAnimation on rotation {
            loops: Animation.Infinite
            from: 0
            to: -360
            running: pdfModel.loading
            duration: 1500
        }
    }

    Shortcut {
        sequence: "space"
        enabled: listView.visible
        onActivated: {
            if(listView.currentIndex >= listView.count - 1)
                listView.positionViewAtIndex(0, ListView.Beginning);
            else if(animation)
                listView.currentIndex++;
            else
                listView.positionViewAtIndex(listView.currentIndex+1, ListView.Beginning);
        }
    }
}
